from .logging import LogLevel, initialize_logging
from .metrics import Metrics
from .retry import retry
from .timeout import timeout

__all__ = ["LogLevel", "Metrics", "initialize_logging", "retry", "timeout"]



from pytest import fixture

from homeconnect_watcher.db import WatcherDBClient
from homeconnect_watcher.event import HomeConnectEvent


@fixture
def db_with_events(db_client: WatcherDBClient, request) -> WatcherDBClient:
    marker = request.node.get_closest_marker("events")
    if marker is not None:
        db_client.write_events(
            [HomeConnectEvent.from_string(line) for line in marker.args[0].split("\n") if len(line) > 0]
        )
        db_client.refresh_views()
    return db_client



from authlib.oauth2.rfc6749 import OAuth2Token
from pytest import mark

from homeconnect_watcher.client import HomeConnectAppliance, HomeConnectClient


class TestClientToken:
    def test_save_token(self, client: HomeConnectClient):
        assert client.token_cache.is_file()

    def test_load_token(self, client: HomeConnectClient):
        token = client._load_token()
        assert isinstance(token, OAuth2Token)


class TestAuthorize:
    def test_authorization_url(self, client: HomeConnectClient):
        assert client.authorization_url.startswith("https://")


class TestAppliances:
    @mark.asyncio
    async def test_appliances(self, client: HomeConnectClient):
        appliances = await client.appliances
        assert len(appliances) > 0
        assert all(isinstance(app, HomeConnectAppliance) for app in appliances)

    @mark.asyncio
    async def test_get_appliance(self, client: HomeConnectClient):
        appliance = (await client.appliances)[0]
        assert await client.get_appliance(appliance.appliance_id) == appliance

    @mark.asyncio
    async def test_refresh_appliances(self, client: HomeConnectClient):
        client._appliances = []
        assert len(await client.appliances) == 0
        await client.refresh_appliances()
        assert len(await client.appliances) >= 0



from homeconnect_watcher.db import WatcherDBClient
from homeconnect_watcher.event import HomeConnectEvent


class TestDBClient:
    def test_event_count(self, db_client: WatcherDBClient) -> None:
        assert db_client.event_count == 0
        db_client.write_events([HomeConnectEvent(timestamp=1674291950, event="KEEP-ALIVE")])
        assert db_client.event_count == 1

    def test_create_views_when_exist(self, db_client: WatcherDBClient):
        db_client.create_views()  # This should not cause any problems

    def test_drop_views(self, db_client: WatcherDBClient):
        db_client.drop_views()

    def test_refresh_views(self, db_client: WatcherDBClient):
        db_client.refresh_views()



from pytest import mark

from homeconnect_watcher.db import WatcherDBClient


@mark.events(
    """{"appliance_id": "SIEMENS-EX877LVV5E-AB1234567890", "event": "NOTIFY", "timestamp": 1703264021, "data": {"haId": "SIEMENS-EX877LVV5E-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Root.SelectedProgram", "level": "hint", "timestamp": 1703264019, "uri": "/api/homeappliances/SIEMENS-EX877LVV5E-AB1234567890/programs/selected", "value": "Cooking.Hob.Program.PowerLevelMode"}]}}"""
)
def test_hob(db_with_events: WatcherDBClient):
    db_with_events.cursor.execute("SELECT * FROM v_appliances")
    result = db_with_events.cursor.fetchall()
    assert len(result) == 1
    assert result[0] == ("SIEMENS-EX877LVV5E-AB1234567890", "Hob")


@mark.events(
    """{"appliance_id": "SIEMENS-TI9553X1RW-AB1234567890", "event": "NOTIFY", "timestamp": 1704095918, "data": {"haId": "SIEMENS-TI9553X1RW-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Root.SelectedProgram", "level": "hint", "timestamp": 1704095918, "uri": "/api/homeappliances/SIEMENS-TI9553X1RW-AB1234567890/programs/selected", "value": null}]}}"""
)
def test_missing_program(db_with_events: WatcherDBClient):
    """Verify that notifications where no program is selected do not show up."""
    db_with_events.cursor.execute("SELECT * FROM v_appliances")
    result = db_with_events.cursor.fetchall()
    assert len(result) == 0


@mark.events(
    """
{"appliance_id": "SIEMENS-TI9553X1RW-AB1234567890", "event": "NOTIFY", "timestamp": 1704095918, "data": {"haId": "SIEMENS-TI9553X1RW-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Root.SelectedProgram", "level": "hint", "timestamp": 1704095918, "uri": "/api/homeappliances/SIEMENS-TI9553X1RW-AB1234567890/programs/selected", "value": null}]}}
{"appliance_id": "SIEMENS-TI9553X1RW-AB1234567890", "event": "NOTIFY", "timestamp": 1708587519, "data": {"haId": "SIEMENS-TI9553X1RW-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Root.SelectedProgram", "level": "hint", "timestamp": 1708587519, "uri": "/api/homeappliances/SIEMENS-TI9553X1RW-AB1234567890/programs/selected", "value": "ConsumerProducts.CoffeeMaker.Program.Beverage.Coffee"}, {"handling": "none", "key": "ConsumerProducts.CoffeeMaker.Option.BeanAmount", "level": "hint", "timestamp": 1708587519, "uri": "/api/homeappliances/SIEMENS-TI9553X1RW-AB1234567890/programs/selected/options/ConsumerProducts.CoffeeMaker.Option.BeanAmount", "value": "ConsumerProducts.CoffeeMaker.EnumType.BeanAmount.Strong"}, {"handling": "none", "key": "ConsumerProducts.CoffeeMaker.Option.FillQuantity", "level": "hint", "timestamp": 1708587519, "unit": "ml", "uri": "/api/homeappliances/SIEMENS-TI9553X1RW-AB1234567890/programs/selected/options/ConsumerProducts.CoffeeMaker.Option.FillQuantity", "value": 140}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707925134, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "LaundryCare.Common.Option.VarioPerfect", "level": "hint", "timestamp": 1707925131, "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/selected/options/LaundryCare.Common.Option.VarioPerfect", "value": "LaundryCare.Common.EnumType.VarioPerfect.Off"}, {"handling": "none", "key": "LaundryCare.Washer.Option.SpinSpeed", "level": "hint", "timestamp": 1707925131, "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/selected/options/LaundryCare.Washer.Option.SpinSpeed", "value": "LaundryCare.Washer.EnumType.SpinSpeed.RPM1200"}, {"handling": "none", "key": "LaundryCare.Washer.Option.Temperature", "level": "hint", "timestamp": 1707925131, "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/selected/options/LaundryCare.Washer.Option.Temperature", "value": "LaundryCare.Washer.EnumType.Temperature.GC40"}, {"handling": "none", "key": "BSH.Common.Root.SelectedProgram", "level": "hint", "timestamp": 1707925131, "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/selected", "value": "LaundryCare.Washer.Program.DelicatesSilk"}]}}
"""
)
def test_multiple(db_with_events: WatcherDBClient):
    db_with_events.cursor.execute("SELECT * FROM v_appliances")
    result = db_with_events.cursor.fetchall()
    assert len(result) == 2



from pytest import fixture

from homeconnect_watcher.event import HomeConnectEvent
from homeconnect_watcher.exporter.postgres import PGExporter


class TestPGExporter:
    @fixture(scope="function")
    def exporter(self, postgresql):
        return PGExporter(
            connection_string=f"postgresql://{postgresql.info.user}:@{postgresql.info.host}:{postgresql.info.port}/{postgresql.info.dbname}"
        )

    @fixture(scope="class")
    def event(self) -> HomeConnectEvent:
        return HomeConnectEvent(
            appliance_id="SIEMENS-EX877LVV5E-AB1234567890",
            event="STATUS-REQUEST",
            timestamp=1641970292,
            data={
                "status": [
                    {"key": "BSH.Common.Status.LocalControlActive", "value": False},
                    {"key": "BSH.Common.Status.OperationState", "value": "BSH.Common.EnumType.OperationState.Inactive"},
                    {"key": "BSH.Common.Status.RemoteControlActive", "value": True},
                ]
            },
        )

    def test_event_written(self, exporter: PGExporter, event: HomeConnectEvent):
        with exporter:
            exporter.export(event)
        with exporter:
            exporter.cursor.execute("SELECT COUNT(*) FROM events")
            assert exporter.cursor.fetchone() == (1,)

    def test_no_duplicates(self, exporter: PGExporter, event: HomeConnectEvent):
        with exporter:
            exporter.export(event)
        with exporter:
            exporter.export(event)
        with exporter:
            exporter.cursor.execute("SELECT COUNT(*) FROM events")
            assert exporter.cursor.fetchone() == (1,)



from datetime import datetime, timedelta

from homeconnect_watcher.event import HomeConnectEvent
from homeconnect_watcher.exporter import BaseExporter
from homeconnect_watcher.db import WatcherDBClient


class PGExporter(BaseExporter, WatcherDBClient):
    def __init__(self, connection_string: str, refresh_interval: timedelta = timedelta(hours=6)):
        WatcherDBClient.__init__(self, connection_string=connection_string)
        BaseExporter.__init__(self)
        self.refresh_interval = refresh_interval
        self._next_refresh: datetime = datetime.now() + self.refresh_interval

    def __enter__(self) -> "PGExporter":
        WatcherDBClient.__enter__(self)
        return self

    def __exit__(self, exc_type, exc_val, exc_tb) -> None:
        WatcherDBClient.__exit__(self, exc_type, exc_val, exc_tb)

    def export(self, event: HomeConnectEvent) -> None:
        self.write_events([event])
        if datetime.now() > self._next_refresh:
            self.refresh_views()
            self._next_refresh: datetime = datetime.now() + self.refresh_interval



from datetime import datetime, timedelta
from pathlib import Path

from pytest import fixture
from pytest_mock.plugin import MockerFixture

from homeconnect_watcher.exporter.file import FileExporter


class TestFileExporter:
    @fixture(scope="function")
    def exporter(self, tmp_path: Path):
        exporter = FileExporter(tmp_path, flush_interval=timedelta(minutes=5))
        return exporter

    def test_file_opened(self, exporter: FileExporter):
        assert len(list(exporter.path.glob("hcw*.jsonl"))) == 0
        with exporter:
            assert len(list(exporter.path.glob("hcw*.jsonl"))) == 1

    def test_flush_skipped(self, exporter: FileExporter, mocker: MockerFixture):
        mock = mocker.Mock()
        exporter._fp = mock
        exporter._flush()
        mock.flush.assert_not_called()

    def test_flushed(self, exporter: FileExporter, mocker: MockerFixture):
        mock = mocker.Mock()
        exporter._fp = mock
        exporter._last_flush = datetime.now() - timedelta(minutes=10)
        exporter._flush()
        mock.flush.assert_called_once()

    def test_repeated_flush_skipped(self, exporter: FileExporter, mocker: MockerFixture):
        mock = mocker.Mock()
        exporter._fp = mock
        exporter._last_flush = datetime.now() - timedelta(minutes=10)
        exporter._flush()
        exporter._flush()
        mock.flush.assert_called_once()



from importlib.resources import files
from re import IGNORECASE, search


class View:
    def __init__(self, query: str):
        self.query = query

    @property
    def drop_query(self) -> str:
        return f"DROP {'MATERIALIZED' if self.materialized else ''} VIEW IF EXISTS {self.name}"

    @property
    def materialized(self) -> bool:
        return "MATERIALIZED VIEW" in self.query

    @property
    def name(self) -> str:
        result = search("VIEW(?: IF NOT EXISTS)? ([a-z_]+) AS", self.query, IGNORECASE)
        return result.group(1)


def load_views() -> list[View]:
    resources = [r for r in (files("homeconnect_watcher") / "sql").iterdir() if r.is_file()]
    return [View(resource.read_text()) for resource in sorted(resources)]


def load_view(name: str) -> View:
    for view in load_views():
        if view.name == name:
            return view
    raise KeyError(f"No such view: {name}")



from psycopg import Connection


def clean_schema(connection: Connection) -> None:
    with connection.cursor() as cursor:
        db_name = connection.info.dbname
        cursor.execute(f"DROP SCHEMA IF EXISTS {db_name} CASCADE; CREATE SCHEMA {db_name};")



from homeconnect_watcher.db.view import load_view, load_views


def test_view_names():
    views = load_views()
    for view in views:
        assert len(view.name) > 0


def test_materialized():
    assert not load_view("v_appliances").materialized
    assert load_view("v_sessions").materialized



from datetime import datetime
from pytest import mark

from homeconnect_watcher.db import WatcherDBClient


@mark.events(
    """{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "EVENT", "timestamp": 1703406303, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "acknowledge", "key": "BSH.Common.Event.ProgramFinished", "level": "hint", "timestamp": 1703406303, "value": "BSH.Common.EnumType.EventPresentState.Present"}]}}"""
)
def test_single_event(db_with_events: WatcherDBClient):
    db_with_events.cursor.execute("SELECT * FROM v_events")
    result = db_with_events.cursor.fetchall()
    assert len(result) == 1
    assert result[0] == ("SIEMENS-WM14T6H9NL-AB1234567890", "ProgramFinished", datetime(2023, 12, 24, 9, 25, 3), None)


@mark.events(
    """
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "EVENT", "timestamp": 1703406303, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "acknowledge", "key": "BSH.Common.Event.ProgramFinished", "level": "hint", "timestamp": 1703406303, "value": "BSH.Common.EnumType.EventPresentState.Present"}]}}
{"appliance_id": "SIEMENS-EX877LVV5E-AB1234567890", "event": "EVENT", "timestamp": 1703412414, "data": {"haId": "SIEMENS-EX877LVV5E-AB1234567890", "items": [{"handling": "acknowledge", "key": "BSH.Common.Event.ProgramFinished", "level": "hint", "timestamp": 1703412414, "value": "BSH.Common.EnumType.EventPresentState.Present"}]}}
{"appliance_id": "SIEMENS-EX877LVV5E-AB1234567890", "event": "EVENT", "timestamp": 1703412423, "data": {"haId": "SIEMENS-EX877LVV5E-AB1234567890", "items": [{"handling": "acknowledge", "key": "BSH.Common.Event.ProgramFinished", "level": "hint", "timestamp": 1703412423, "value": "BSH.Common.EnumType.EventPresentState.Off"}]}}
"""
)
def test_multiple_appliances(db_with_events: WatcherDBClient):
    db_with_events.cursor.execute("SELECT * FROM v_events")
    result = db_with_events.cursor.fetchall()
    assert len(result) == 2
    assert result[0] == ("SIEMENS-WM14T6H9NL-AB1234567890", "ProgramFinished", datetime(2023, 12, 24, 9, 25, 3), None)
    assert result[1] == (
        "SIEMENS-EX877LVV5E-AB1234567890",
        "ProgramFinished",
        datetime(2023, 12, 24, 11, 6, 54),
        datetime(2023, 12, 24, 11, 7, 3),
    )


@mark.events(
    """
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "EVENT", "timestamp": 1707376385, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "acknowledge", "key": "BSH.Common.Event.ProgramFinished", "level": "hint", "timestamp": 1707376385, "value": "BSH.Common.EnumType.EventPresentState.Present"}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "EVENT", "timestamp": 1707376691, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "acknowledge", "key": "BSH.Common.Event.ProgramFinished", "level": "hint", "timestamp": 1707376691, "value": "BSH.Common.EnumType.EventPresentState.Off"}]}}
{"appliance_id": "SIEMENS-TI9553X1RW-AB1234567890", "event": "EVENT", "timestamp": 1707377425, "data": {"haId": "SIEMENS-TI9553X1RW-AB1234567890", "items": [{"handling": "none", "key": "ConsumerProducts.CoffeeMaker.Event.DripTrayFull", "level": "alert", "timestamp": 1707377425, "value": "BSH.Common.EnumType.EventPresentState.Present"}]}}
{"appliance_id": "SIEMENS-TI9553X1RW-AB1234567890", "event": "EVENT", "timestamp": 1707377768, "data": {"haId": "SIEMENS-TI9553X1RW-AB1234567890", "items": [{"handling": "none", "key": "ConsumerProducts.CoffeeMaker.Event.DripTrayFull", "level": "alert", "timestamp": 1707377768, "value": "BSH.Common.EnumType.EventPresentState.Off"}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "EVENT", "timestamp": 1707379832, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "acknowledge", "key": "BSH.Common.Event.ProgramFinished", "level": "hint", "timestamp": 1707379832, "value": "BSH.Common.EnumType.EventPresentState.Present"}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "EVENT", "timestamp": 1707379944, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "acknowledge", "key": "BSH.Common.Event.ProgramFinished", "level": "hint", "timestamp": 1707379944, "value": "BSH.Common.EnumType.EventPresentState.Off"}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "EVENT", "timestamp": 1707382275, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "acknowledge", "key": "BSH.Common.Event.ProgramFinished", "level": "hint", "timestamp": 1707382275, "value": "BSH.Common.EnumType.EventPresentState.Present"}]}}
"""
)
def test_successive(db_with_events: WatcherDBClient):
    db_with_events.cursor.execute("SELECT * FROM v_events")
    result = db_with_events.cursor.fetchall()
    assert len(result) == 4
    assert result[0] == (
        "SIEMENS-WM14T6H9NL-AB1234567890",
        "ProgramFinished",
        datetime(2024, 2, 8, 8, 13, 5),
        datetime(2024, 2, 8, 8, 18, 11),
    )
    assert result[1] == (
        "SIEMENS-TI9553X1RW-AB1234567890",
        "DripTrayFull",
        datetime(2024, 2, 8, 8, 30, 25),
        datetime(2024, 2, 8, 8, 36, 8),
    )
    assert result[2] == (
        "SIEMENS-WM14T6H9NL-AB1234567890",
        "ProgramFinished",
        datetime(2024, 2, 8, 9, 10, 32),
        datetime(2024, 2, 8, 9, 12, 24),
    )
    assert result[3] == (
        "SIEMENS-WM14T6H9NL-AB1234567890",
        "ProgramFinished",
        datetime(2024, 2, 8, 9, 51, 15),
        None,
    )


@mark.events(
    """
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "EVENT", "timestamp": 1707466277, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "acknowledge", "key": "BSH.Common.Event.ProgramFinished", "level": "hint", "timestamp": 1707466277, "value": "BSH.Common.EnumType.EventPresentState.Present"}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "DISCONNECTED", "timestamp": 1707490575, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "handling": "none", "key": "BSH.Common.Appliance.Disconnected", "level": "hint", "timestamp": 1707490575, "value": true}}
"""
)
def test_end_by_disconnect(db_with_events: WatcherDBClient):
    db_with_events.cursor.execute("SELECT * FROM v_events")
    result = db_with_events.cursor.fetchall()
    assert len(result) == 1
    assert result[0] == (
        "SIEMENS-WM14T6H9NL-AB1234567890",
        "ProgramFinished",
        datetime(2024, 2, 9, 9, 11, 17),
        datetime(2024, 2, 9, 15, 56, 15),
    )



from datetime import datetime
from pytest import mark

from homeconnect_watcher.db import WatcherDBClient


class TestCoffeeMaker:
    pass


class TestWasher:
    @mark.events(
        """
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "CONNECTED", "timestamp": 1704972036, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "handling": "none", "key": "BSH.Common.Appliance.Connected", "level": "hint", "timestamp": 1704972036, "value": true}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "STATUS-REQUEST", "timestamp": 1704972038, "data": {"status": [{"key": "BSH.Common.Status.DoorState", "value": "BSH.Common.EnumType.DoorState.Locked"}, {"key": "BSH.Common.Status.LocalControlActive", "value": false}, {"key": "BSH.Common.Status.OperationState", "value": "BSH.Common.EnumType.OperationState.Run"}, {"key": "BSH.Common.Status.RemoteControlActive", "value": true}, {"key": "BSH.Common.Status.RemoteControlStartAllowed", "value": false}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "SETTINGS-REQUEST", "timestamp": 1704972040, "data": {"settings": [{"key": "BSH.Common.Setting.PowerState", "value": "BSH.Common.EnumType.PowerState.On"}, {"key": "LaundryCare.Washer.Setting.IDos1BaseLevel", "value": 30, "unit": "ml"}, {"key": "LaundryCare.Washer.Setting.IDos2BaseLevel", "value": 30, "unit": "ml"}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "ACTIVE-PROGRAM-REQUEST", "timestamp": 1704972041, "data": {"constraints": {"access": "read"}, "key": "LaundryCare.Washer.Program.Cotton", "options": [{"key": "LaundryCare.Common.Option.LoadRecommendation", "value": 8000, "unit": "gram"}, {"key": "LaundryCare.Common.Option.VarioPerfect", "value": "LaundryCare.Common.EnumType.VarioPerfect.Off"}, {"key": "LaundryCare.Washer.Option.IDos1DosingLevel", "value": "LaundryCare.Washer.EnumType.IDosingLevel.Normal"}, {"key": "LaundryCare.Washer.Option.IDos2DosingLevel", "value": "LaundryCare.Washer.EnumType.IDosingLevel.Light"}, {"key": "LaundryCare.Washer.Option.LessIroning", "value": false}, {"key": "LaundryCare.Washer.Option.Prewash", "value": false}, {"key": "LaundryCare.Washer.Option.SpinSpeed", "value": "LaundryCare.Washer.EnumType.SpinSpeed.RPM1200"}, {"key": "LaundryCare.Washer.Option.Temperature", "value": "LaundryCare.Washer.EnumType.Temperature.GC60"}, {"key": "LaundryCare.Washer.Option.WaterAndRinsePlus", "value": "LaundryCare.Washer.EnumType.WaterAndRinsePlus.Off"}, {"key": "BSH.Common.Option.ProgramProgress", "value": 0, "unit": "%"}, {"key": "BSH.Common.Option.RemainingProgramTime", "value": 8277, "unit": "seconds"}, {"key": "BSH.Common.Option.RemainingProgramTimeIsEstimated", "value": true}, {"key": "LaundryCare.Washer.Option.ProcessPhase", "value": "LaundryCare.Washer.EnumType.ProcessPhase.Washing"}, {"key": "BSH.Common.Option.EnergyForecast", "value": 80, "unit": "%"}, {"key": "BSH.Common.Option.WaterForecast", "value": 60, "unit": "%"}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "SELECTED-PROGRAM-REQUEST", "timestamp": 1704972043, "data": {"constraints": {"access": "read"}, "key": "LaundryCare.Washer.Program.Cotton", "options": [{"key": "LaundryCare.Common.Option.LoadRecommendation", "value": 8000, "unit": "gram"}, {"key": "LaundryCare.Common.Option.VarioPerfect", "value": "LaundryCare.Common.EnumType.VarioPerfect.Off"}, {"key": "LaundryCare.Washer.Option.IDos1DosingLevel", "value": "LaundryCare.Washer.EnumType.IDosingLevel.Normal"}, {"key": "LaundryCare.Washer.Option.IDos2DosingLevel", "value": "LaundryCare.Washer.EnumType.IDosingLevel.Light"}, {"key": "LaundryCare.Washer.Option.LessIroning", "value": false}, {"key": "LaundryCare.Washer.Option.Prewash", "value": false}, {"key": "LaundryCare.Washer.Option.SpinSpeed", "value": "LaundryCare.Washer.EnumType.SpinSpeed.RPM1200"}, {"key": "LaundryCare.Washer.Option.Temperature", "value": "LaundryCare.Washer.EnumType.Temperature.GC60"}, {"key": "LaundryCare.Washer.Option.WaterAndRinsePlus", "value": "LaundryCare.Washer.EnumType.WaterAndRinsePlus.Off"}, {"key": "BSH.Common.Option.RemainingProgramTime", "value": 8276, "unit": "seconds"}, {"key": "BSH.Common.Option.RemainingProgramTimeIsEstimated", "value": true}, {"key": "BSH.Common.Option.EnergyForecast", "value": 80, "unit": "%"}, {"key": "BSH.Common.Option.WaterForecast", "value": 60, "unit": "%"}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "STATUS", "timestamp": 1704972043, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Status.LocalControlActive", "level": "hint", "timestamp": 1704972036, "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/status/BSH.Common.Status.LocalControlActive", "value": false}, {"handling": "none", "key": "BSH.Common.Status.RemoteControlActive", "level": "hint", "timestamp": 1704972036, "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/status/BSH.Common.Status.RemoteControlActive", "value": true}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1704972093, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.RemainingProgramTime", "level": "hint", "timestamp": 1704972093, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.RemainingProgramTime", "value": 8220}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1704974129, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.RemainingProgramTime", "level": "hint", "timestamp": 1704974129, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.RemainingProgramTime", "value": 4080}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1704976349, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.RemainingProgramTime", "level": "hint", "timestamp": 1704976349, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.RemainingProgramTime", "value": 1860}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1704979640, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.RemainingProgramTime", "level": "hint", "timestamp": 1704979640, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.RemainingProgramTime", "value": 0}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "STATUS", "timestamp": 1704979641, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Status.OperationState", "level": "hint", "timestamp": 1704979641, "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/status/BSH.Common.Status.OperationState", "value": "BSH.Common.EnumType.OperationState.Finished"}, {"handling": "none", "key": "BSH.Common.Status.DoorState", "level": "hint", "timestamp": 1704979641, "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/status/BSH.Common.Status.DoorState", "value": "BSH.Common.EnumType.DoorState.Closed"}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "EVENT", "timestamp": 1704979641, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "acknowledge", "key": "BSH.Common.Event.ProgramFinished", "level": "hint", "timestamp": 1704979641, "value": "BSH.Common.EnumType.EventPresentState.Present"}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1704979642, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1704979642, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 0}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "DISCONNECTED", "timestamp": 1704993381, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "handling": "none", "key": "BSH.Common.Appliance.Disconnected", "level": "hint", "timestamp": 1704993381, "value": true}}
"""
    )
    def test_simple_session(self, db_with_events: WatcherDBClient):
        db_with_events.cursor.execute(
            "SELECT appliance_id, trigger_time, start_time, end_time, program FROM v_sessions"
        )
        result = db_with_events.cursor.fetchall()
        assert len(result) == 1
        assert result[0] == (
            "SIEMENS-WM14T6H9NL-AB1234567890",
            datetime(2024, 1, 11, 12, 20, 38),
            datetime(2024, 1, 11, 12, 20, 38),
            datetime(2024, 1, 11, 14, 27, 20),
            "Cotton",
        )

    @mark.events(
        """
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707382868, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707382868, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/selected/options/BSH.Common.Option.FinishInRelative", "value": 84600}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "STATUS", "timestamp": 1707382871, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Status.DoorState", "level": "hint", "timestamp": 1707382871, "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/status/BSH.Common.Status.DoorState", "value": "BSH.Common.EnumType.DoorState.Locked"}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "STATUS", "timestamp": 1707382872, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Status.OperationState", "level": "hint", "timestamp": 1707382872, "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/status/BSH.Common.Status.OperationState", "value": "BSH.Common.EnumType.OperationState.DelayedStart"}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707382872, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.RemainingProgramTime", "level": "hint", "timestamp": 1707382872, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.RemainingProgramTime", "value": 12660}, {"handling": "none", "key": "BSH.Common.Root.ActiveProgram", "level": "hint", "timestamp": 1707382872, "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active", "value": "LaundryCare.Washer.Program.Cotton.CottonEco"}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "STATUS", "timestamp": 1707382874, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Status.LocalControlActive", "level": "hint", "timestamp": 1707382874, "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/status/BSH.Common.Status.LocalControlActive", "value": false}, {"handling": "none", "key": "BSH.Common.Status.RemoteControlActive", "level": "hint", "timestamp": 1707382874, "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/status/BSH.Common.Status.RemoteControlActive", "value": true}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707382931, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707382931, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 84540}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707382991, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707382991, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 84480}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707383051, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707383051, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 84420}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "STATUS-REQUEST", "timestamp": 1707383053, "data": {"status": [{"key": "BSH.Common.Status.DoorState", "value": "BSH.Common.EnumType.DoorState.Locked"}, {"key": "BSH.Common.Status.LocalControlActive", "value": false}, {"key": "BSH.Common.Status.OperationState", "value": "BSH.Common.EnumType.OperationState.DelayedStart"}, {"key": "BSH.Common.Status.RemoteControlActive", "value": true}, {"key": "BSH.Common.Status.RemoteControlStartAllowed", "value": false}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707383111, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707383111, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 84360}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707384131, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707384131, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 83340}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707385510, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707385510, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 81960}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707386769, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707386769, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 80700}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707388088, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707388088, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 79380}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707389348, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707389348, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 78120}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707391986, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707391986, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 75480}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707393245, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707393245, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 74220}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707394564, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707394564, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 72900}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707395824, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707395823, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 71640}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707398342, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707398342, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 69120}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707399661, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707399661, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 67800}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707400980, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707400980, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 66480}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707402239, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707402239, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 65220}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707403559, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707403559, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 63900}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707404818, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707404818, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 62640}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707406137, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707406137, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 61320}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707407456, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707407456, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 60000}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707408715, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707408715, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 58740}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707410094, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707410094, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 57360}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707411414, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707411414, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 56040}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707413452, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707413452, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 54000}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707415431, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707415431, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 52020}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707417410, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707417410, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 50040}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707419449, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707419449, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 48000}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707421427, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707421427, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 46020}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707423406, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707423406, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 44040}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707425445, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707425445, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 42000}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707427424, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707427424, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 40020}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707429403, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707429403, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 38040}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707431441, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707431441, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 36000}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707433420, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707433420, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 34020}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707435399, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707435399, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 32040}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707437438, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707437438, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 30000}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707439417, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707439416, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 28020}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707441395, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707441395, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 26040}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707443434, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707443434, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 24000}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707445293, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707445293, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 22140}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707447392, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707447392, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 20040}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707449251, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707449251, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 18180}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707451409, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707451409, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 16020}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707453388, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707453388, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 14040}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "STATUS", "timestamp": 1707453868, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Status.OperationState", "level": "hint", "timestamp": 1707453868, "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/status/BSH.Common.Status.OperationState", "value": "BSH.Common.EnumType.OperationState.Run"}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707454527, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.RemainingProgramTime", "level": "hint", "timestamp": 1707454527, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.RemainingProgramTime", "value": 12000}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707456159, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.RemainingProgramTime", "level": "hint", "timestamp": 1707456159, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.RemainingProgramTime", "value": 10020}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707458138, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.RemainingProgramTime", "level": "hint", "timestamp": 1707458138, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.RemainingProgramTime", "value": 8040}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707460177, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.RemainingProgramTime", "level": "hint", "timestamp": 1707460177, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.RemainingProgramTime", "value": 6000}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707462096, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.RemainingProgramTime", "level": "hint", "timestamp": 1707462096, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.RemainingProgramTime", "value": 4080}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707464291, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.RemainingProgramTime", "level": "hint", "timestamp": 1707464291, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.RemainingProgramTime", "value": 2040}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707466275, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.RemainingProgramTime", "level": "hint", "timestamp": 1707466275, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.RemainingProgramTime", "value": 0}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "EVENT", "timestamp": 1707466275, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "acknowledge", "key": "LaundryCare.Washer.Event.IDos1FillLevelPoor", "level": "warning", "timestamp": 1707466275, "value": "BSH.Common.EnumType.EventPresentState.Present"}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "EVENT", "timestamp": 1707466277, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "acknowledge", "key": "BSH.Common.Event.ProgramFinished", "level": "hint", "timestamp": 1707466277, "value": "BSH.Common.EnumType.EventPresentState.Present"}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "STATUS", "timestamp": 1707466277, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Status.OperationState", "level": "hint", "timestamp": 1707466277, "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/status/BSH.Common.Status.OperationState", "value": "BSH.Common.EnumType.OperationState.Finished"}, {"handling": "none", "key": "BSH.Common.Status.DoorState", "level": "hint", "timestamp": 1707466277, "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/status/BSH.Common.Status.DoorState", "value": "BSH.Common.EnumType.DoorState.Closed"}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707466277, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.FinishInRelative", "level": "hint", "timestamp": 1707466277, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active/options/BSH.Common.Option.FinishInRelative", "value": 0}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "EVENT", "timestamp": 1707467283, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "acknowledge", "key": "BSH.Common.Event.ProgramFinished", "level": "hint", "timestamp": 1707467283, "value": "BSH.Common.EnumType.EventPresentState.Off"}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "STATUS-REQUEST", "timestamp": 1707467287, "data": {"status": [{"key": "BSH.Common.Status.DoorState", "value": "BSH.Common.EnumType.DoorState.Open"}, {"key": "BSH.Common.Status.LocalControlActive", "value": true}, {"key": "BSH.Common.Status.OperationState", "value": "BSH.Common.EnumType.OperationState.Ready"}, {"key": "BSH.Common.Status.RemoteControlActive", "value": false}, {"key": "BSH.Common.Status.RemoteControlStartAllowed", "value": false}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "STATUS", "timestamp": 1707467287, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Status.OperationState", "level": "hint", "timestamp": 1707467283, "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/status/BSH.Common.Status.OperationState", "value": "BSH.Common.EnumType.OperationState.Ready"}, {"handling": "none", "key": "BSH.Common.Status.DoorState", "level": "hint", "timestamp": 1707467283, "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/status/BSH.Common.Status.DoorState", "value": "BSH.Common.EnumType.DoorState.Open"}, {"handling": "none", "key": "BSH.Common.Status.LocalControlActive", "level": "hint", "timestamp": 1707467283, "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/status/BSH.Common.Status.LocalControlActive", "value": true}, {"handling": "none", "key": "BSH.Common.Status.RemoteControlActive", "level": "hint", "timestamp": 1707467283, "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/status/BSH.Common.Status.RemoteControlActive", "value": false}]}}
{"appliance_id": "SIEMENS-WM14T6H9NL-AB1234567890", "event": "NOTIFY", "timestamp": 1707467287, "data": {"haId": "SIEMENS-WM14T6H9NL-AB1234567890", "items": [{"handling": "none", "key": "BSH.Common.Option.EstimatedTotalProgramTime", "level": "hint", "timestamp": 1707467283, "unit": "seconds", "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/selected/options/BSH.Common.Option.EstimatedTotalProgramTime", "value": 0}, {"handling": "none", "key": "BSH.Common.Root.ActiveProgram", "level": "hint", "timestamp": 1707467283, "uri": "/api/homeappliances/SIEMENS-WM14T6H9NL-AB1234567890/programs/active", "value": null}]}}
"""
    )
    def test_delayed_session(self, db_with_events: WatcherDBClient):
        db_with_events.cursor.execute(
            "SELECT appliance_id, trigger_time, start_time, end_time, program FROM v_sessions"
        )
        result = db_with_events.cursor.fetchall()
        assert len(result) == 1
        assert result[0] == (
            "SIEMENS-WM14T6H9NL-AB1234567890",
            datetime(2024, 2, 8, 10, 1, 12),
            datetime(2024, 2, 9, 5, 44, 28),
            datetime(2024, 2, 9, 9, 11, 15),
            "CottonEco",
        )


class TestDryer:
    pass



from asyncio import run as async_run
from datetime import timedelta
from pathlib import Path
from typing import Annotated, Optional

from fastapi import FastAPI
from fastapi.responses import Response
from typer import Option, Typer
from uvicorn import run

from homeconnect_watcher.api import loop
from homeconnect_watcher.client.client import HomeConnectSimulationClient, HomeConnectClient
from homeconnect_watcher.db import WatcherDBClient
from homeconnect_watcher.db.utils import clean_schema
from homeconnect_watcher.exporter.file import FileExporter
from homeconnect_watcher.exporter.postgres import PGExporter
from homeconnect_watcher.read import read_events
from homeconnect_watcher.utils import LogLevel, Metrics, initialize_logging

app = Typer()


@app.command()
def authorize(
    log_level: LogLevel = Option(LogLevel.INFO, envvar="HCW_LOGLEVEL"),
):
    initialize_logging(level=log_level)
    client = HomeConnectClient()

    api = FastAPI()

    @api.get("/code")
    async def code(code: str):
        url = f"{client.client.redirect_uri}/?code={code}&grant_type=authorization_code"
        await client.authorize(url)
        print("Authorized.")
        return Response("OK", 200)

    print("Visit the following URL:")
    print(client.authorization_url)
    run(api, host="0.0.0.0", port=8000)


@app.command()
def watch(
    simulation: bool = False,
    flush_interval: int = Option(300, envvar="HCW_FLUSH_INTERVAL"),
    log_level: LogLevel = Option(LogLevel.INFO, envvar="HCW_LOGLEVEL"),
    metrics_port: Optional[int] = Option(None, envvar="HCW_METRICS_PORT"),
    log_path: Annotated[Optional[str], Option(envvar="HOMECONNECT_PATH")] = None,
    db_uri: Annotated[Optional[str], Option(envvar="HCW_DB_URI")] = None,
):
    initialize_logging(level=log_level)
    metrics = Metrics(port=metrics_port) if metrics_port else None
    client = (HomeConnectSimulationClient if simulation else HomeConnectClient)(metrics=metrics)
    exporters = []
    if log_path is not None:
        exporters.append(FileExporter(path=Path(log_path), flush_interval=timedelta(seconds=flush_interval)))
    if db_uri is not None:
        exporters.append(PGExporter(connection_string=db_uri))
    async_run(loop(client, exporters))


@app.command()
def load(
    db_uri: Annotated[Optional[str], Option(envvar="HCW_DB_URI")] = None,
    log_path: Annotated[Optional[str], Option(envvar="HOMECONNECT_PATH")] = None,
    clean: bool = False,
):
    exporter = PGExporter(connection_string=db_uri)
    if clean:
        with exporter:
            clean_schema(connection=exporter.connection)
    with exporter:
        count_before = exporter.event_count
        n_events = 0
        for events in read_events(Path(log_path)):
            n_events += len(events)
            exporter.bulk_export(events)
        print(f"Added {exporter.event_count - count_before} of {n_events} events.")


@app.command()
def views(db_uri: Annotated[Optional[str], Option(envvar="HCW_DB_URI")] = None, drop: bool = False):
    with WatcherDBClient(connection_string=db_uri, init=False) as client:
        with client.connection.transaction():
            if drop:
                client.drop_views()
            client.create_views()


@app.command()
def refresh_view(db_uri: Annotated[Optional[str], Option(envvar="HCW_DB_URI")] = None):
    with WatcherDBClient(connection_string=db_uri) as client:
        client.refresh_views()



from pytest import mark, skip

from homeconnect_watcher.client import HomeConnectAppliance
from homeconnect_watcher.trigger import Trigger
from homeconnect_watcher.event import HomeConnectEvent


class TestFromStream:
    def test_keep_alive(self, stream_event: bytes):
        if b"KEEP-ALIVE" not in stream_event:
            skip()
        event = HomeConnectEvent.from_stream(stream_event)
        assert event.event == "KEEP-ALIVE"
        assert event.data is None
        assert event.appliance_id is None
        assert event.timestamp is not None

    def test_status(self, stream_event: bytes):
        if b"STATUS" not in stream_event:
            skip()
        event = HomeConnectEvent.from_stream(stream_event)
        assert event.event == "STATUS"
        assert event.data is not None
        assert event.appliance_id == event.data["haId"]
        assert event.timestamp is not None

    def test_event(self, stream_event: bytes):
        if b"EVENT" not in stream_event:
            skip()
        event = HomeConnectEvent.from_stream(stream_event)
        assert event.event == "EVENT"
        assert event.data is not None
        assert event.appliance_id == event.data["haId"]
        assert event.timestamp is not None

    def test_notify(self, stream_event: bytes):
        if b"NOTIFY" not in stream_event:
            skip()
        event = HomeConnectEvent.from_stream(stream_event)
        assert event.event == "NOTIFY"
        assert event.data is not None
        assert event.appliance_id == event.data["haId"]
        assert event.timestamp is not None

    def test_connected(self, stream_event: bytes):
        if b"event:CONNECTED" not in stream_event:
            skip()
        event = HomeConnectEvent.from_stream(stream_event)
        assert event.event == "CONNECTED"
        assert event.data is not None
        assert event.appliance_id == event.data["haId"]
        assert event.timestamp is not None

    def test_disconnected(self, stream_event: bytes):
        if b"DISCONNECTED" not in stream_event:
            skip()
        event = HomeConnectEvent.from_stream(stream_event)
        assert event.event == "DISCONNECTED"
        assert event.data is not None
        assert event.appliance_id == event.data["haId"]
        assert event.timestamp is not None


class TestRequest:
    @mark.asyncio
    async def test_status(self, appliance: HomeConnectAppliance):
        event = await appliance.get_status()
        assert event.event == "STATUS-REQUEST"
        assert event.is_request
        assert event.timestamp is not None
        assert "status" in event.data

    @mark.asyncio
    async def test_settings(self, appliance: HomeConnectAppliance):
        event = await appliance.get_settings()
        assert event.event == "SETTINGS-REQUEST"
        assert event.is_request
        assert event.timestamp is not None
        assert "settings" in event.data

    @mark.asyncio
    async def test_active_program(self, appliance: HomeConnectAppliance):
        # By default, the simulator appliances are inactive.
        event = await appliance.get_active_program()
        assert event.error["key"] == "SDK.Error.NoProgramActive"
        assert event.items["BSH.Common.Root.ActiveProgram"] is None

    @mark.asyncio
    async def test_selected_program(self, appliance: HomeConnectAppliance):
        event = await appliance.get_selected_program()
        assert event.event == "SELECTED-PROGRAM-REQUEST"
        assert event.is_request
        assert event.timestamp is not None
        assert "key" in event.data


class TestEvent:
    def test_items(self, event: HomeConnectEvent) -> None:
        items = event.items
        assert isinstance(items, dict)

    def test_trigger(self, event: HomeConnectEvent) -> None:
        trigger = event.trigger
        if trigger is None:
            assert event.event in ("DISCONNECTED", "STATUS", "KEEP-ALIVE") or event.is_request
        else:
            assert isinstance(trigger, Trigger)



from json import dumps
from logging import getLogger

from psycopg import Connection, Cursor, connect

from .view import load_views
from ..event import HomeConnectEvent

logger = getLogger(__name__)


class WatcherDBClient:
    def __init__(self, connection_string: str, init: bool = True):
        self.connection_string = connection_string
        self.connection: Connection | None = None
        self.cursor: Cursor | None = None
        self.init = init

    def __enter__(self) -> "WatcherDBClient":
        logger.info("Opening database connection.")
        self.connection = connect(self.connection_string, autocommit=True)
        self.cursor = self.connection.cursor()
        if self.init:
            self.create_table()
            self.create_views()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb) -> None:
        self.cursor.close()
        self.cursor = None
        self.connection.close()
        self.connection = None
        logger.info("Database connection closed.")
        return

    @property
    def event_count(self) -> int:
        self.cursor.execute("SELECT COUNT(*) AS cnt FROM events")
        result = self.cursor.fetchone()
        return result[0]

    def create_table(self) -> None:
        """Create the event table."""
        self.cursor.execute(
            """
CREATE TABLE IF NOT EXISTS events (
    appliance_id char(31),
    event varchar(31) NOT NULL,
    timestamp timestamp NOT NULL,
    data jsonb NOT NULL,
    PRIMARY KEY (appliance_id, event, timestamp)
);
"""
        )

    def create_views(self) -> None:
        logger.info("Creating events table.")
        with self.connection.transaction():
            for view in load_views():
                self.connection.execute(view.query)

    def drop_views(self) -> None:
        logger.info("Dropping views.")
        with self.connection.transaction():
            for view in reversed(load_views()):
                self.connection.execute(view.drop_query)

    def refresh_views(self) -> None:
        logger.info("Creating views.")
        with self.connection.transaction():
            for view in load_views():
                if view.materialized:
                    self.connection.execute(f"REFRESH MATERIALIZED VIEW {view.name};")

    def write_events(self, events: list[HomeConnectEvent]) -> None:
        data = [(event.appliance_id or "", event.event, event.datetime, dumps(event.items)) for event in events]
        self.cursor.executemany(
            "INSERT INTO events(appliance_id, event, timestamp, data) VALUES (%s, %s, %s, %s) ON CONFLICT DO NOTHING",
            data,
        )



from pathlib import Path
from typing import Iterable

from tqdm import tqdm

from homeconnect_watcher.event import HomeConnectEvent


def read_events(path: Path) -> Iterable[list[HomeConnectEvent]]:
    for f in tqdm(list(path.glob("*.jsonl"))):
        contents = f.read_text()
        data = []
        for line in contents.split("\n"):
            if len(line) == 0:
                # Skip empty lines
                continue
            event = HomeConnectEvent.from_string(line)
            if event.timestamp is None:
                # Skip events that have no timestamp
                continue
            data.append(event)
        yield data
